package eu.etaxonomy.taxeditor.prototype2.view;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import eu.etaxonomy.taxeditor.prototype2.model.PropertySheetNode;

/**
 * Creates a tree viewer with two columns, one for field names the other
 *  for their values. Note: labels and editing support for values column,
 *  content provider and input for viewer must be set by the calling method 
 * 
 * @author p.ciardelli
 *
 */
public class PropertySheetViewer extends TreeViewer {

	final private TreeViewerColumn propertyFieldColumn;
	private TreeViewerColumn propertyValueColumn;
	private DataBindingContext bindingContext = new DataBindingContext();
	
	public PropertySheetViewer(Composite parent) {
		// Note: SWT.FULL_SELECTION hides lines connecting nodes
		super(parent, SWT.BORDER | SWT.FULL_SELECTION);

		this.getTree().setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, false, true));
//		this.getTree().setLayoutData(new GridData(GridData.FILL_BOTH));
		
		// create a column for property fields
		propertyFieldColumn = new TreeViewerColumn(this, SWT.NONE);
		propertyFieldColumn.getColumn().setMoveable(true);
		propertyFieldColumn.setLabelProvider(new ColumnLabelProvider() {
			public String getText(Object element) {
			    return ((PropertySheetNode) element).getPropertyField();
			}
		});
				
		// create a column for property values		
		propertyValueColumn = new TreeViewerColumn(this, SWT.NONE);
		propertyValueColumn.getColumn().setMoveable(true);
		
		this.getTree().setLinesVisible(true);
		this.getTree().setHeaderVisible(true);
//		this.getTree().getVerticalBar().setEnabled(true);
		this.getTree().getVerticalBar().setEnabled(false);
		
		this.getTree().setSize(200, 300);
		propertyValueColumn.getColumn().setWidth(100);
		propertyFieldColumn.getColumn().setWidth(100);
		
//		try {
//			TableColumnLayout layout = new TableColumnLayout();
//			this.getTree().setLayout(layout);
//			
//			layout.setColumnData( propertyFieldColumn.getColumn(), new ColumnWeightData( 30 ) );
//			layout.setColumnData( propertyValueColumn.getColumn(), new ColumnWeightData( 60 ) );
//		} catch (RuntimeException e) {
//			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}
		
		propertyFieldColumn.getColumn().addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				propertyFieldColumn.getColumn().pack();
			}
		});
		propertyValueColumn.getColumn().addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				propertyValueColumn.getColumn().pack();
			}
		});		
		this.getTree().addListener(SWT.MeasureItem, new Listener() {
			public void handleEvent(Event event) {
//				event.width *= 2;	
			}
		});
		
	}

	public TreeViewerColumn getPropertyFieldColumn() {
		return propertyFieldColumn;
	}

	public TreeViewerColumn getPropertyValueColumn() {
		return propertyValueColumn;
	}

	public DataBindingContext getBindingContext() {
		return bindingContext;
	}
}