// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.bulkeditor.command;

import java.util.Map;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.ContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.CompoundContributionItem;
import org.eclipse.ui.handlers.IHandlerService;

import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineDocumentProvider;
import eu.etaxonomy.taxeditor.annotatedlineeditor.AnnotatedLineEditor;
import eu.etaxonomy.taxeditor.annotatedlineeditor.IEntityCreator;
import eu.etaxonomy.taxeditor.bulkeditor.IBulkEditorConstants;
import eu.etaxonomy.taxeditor.bulkeditor.input.BulkEditorInput;
import eu.etaxonomy.taxeditor.bulkeditor.input.ReferenceEditorInput;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author n.hoffmann
 * @created 17.04.2009
 * @version 1.0
 */
public class DynamicNewObjectMenu extends CompoundContributionItem {
	private static final Logger logger = Logger
			.getLogger(DynamicNewObjectMenu.class);
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.actions.CompoundContributionItem#getContributionItems()
	 */
	@Override
	protected IContributionItem[] getContributionItems() {
		
		return new IContributionItem[] {
				new ContributionItem() {
					public void fill(Menu menu, int index){
						
						final Map<Object, String> classLabelPairs = getClassLabelPairs();
						
						final IHandlerService handlerService = 
										(IHandlerService) PlatformUI.getWorkbench().getService(IHandlerService.class);
						
						for(final Object key : classLabelPairs.keySet()){
							MenuItem menuItem = new MenuItem(menu, -1);
							menuItem.setText(classLabelPairs.get(key));
							menuItem.addSelectionListener(new SelectionListener(){
				
								public void widgetDefaultSelected(SelectionEvent e) {}
				
								public void widgetSelected(SelectionEvent ev) {
									Event event = new Event();
									event.data = key;
									event.text = classLabelPairs.get(key);
									try {
										handlerService.executeCommand(IBulkEditorConstants.DYNAMIC_OPEN_OBJECT_ID, event);
									} catch (Exception e) {
										logger.error("Error executing command", e);
									}
								}				
							});
						}			
					}
				}
		};
	}

	/**
	 * Get class label pairs from Annotated Line Editor's entity creator.
	 * @return
	 */
	private Map<Object, String> getClassLabelPairs() {
		IEditorPart editor = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getActiveEditor();
		if (editor == null || !(editor instanceof AnnotatedLineEditor) || !(((AnnotatedLineEditor) editor).getDocumentProvider() instanceof AnnotatedLineDocumentProvider)) {
			return null;
		}
		IEntityCreator<?> entityCreator = ((AnnotatedLineDocumentProvider) ((AnnotatedLineEditor) editor).getDocumentProvider()).getEntityCreator(editor.getEditorInput());
		return entityCreator.getKeyLabelPairs();
	}	
}