/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.events.IExpansionListener;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.forms.widgets.ToggleHyperlink;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.taxeditor.forms.EditorFormToolkit;

/**
 * @author nho
 *
 */
public abstract class AbstractDescriptionDetailSection extends Section implements IExpansionListener, ModifyListener{

	protected AbstractDescriptionDetailsPage page;
	protected EditorFormToolkit toolkit;
	private Composite client;
	
	/**
	 * @param parent
	 * @param style
	 */
	public AbstractDescriptionDetailSection(Composite parent, AbstractDescriptionDetailsPage page, int sectionStyle) {
		super(parent, Section.TITLE_BAR | SWT.NO_BACKGROUND | sectionStyle);
		toolkit = page.toolkit;
		this.page = page;
				
		this.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		client = toolkit.createComposite(this, SWT.WRAP);
		client.setLayout(new GridLayout());

		createDynamicContents();
		
		this.setClient(client);
		this.addExpansionListener(this);
	}
	
	public Composite getClient(){
		return client;
	}
	
	public abstract void updateSection();
	
	/**
	 * create controls that will change dynamically here
	 */
	protected void createDynamicContents(){
		// default is empty
	}
	
	/**
	 * Call this method after dynamically changing the client area. 
	 * If the options changed is set to true, will also fire a state changed
	 * event to inform the user of unsaved changes. 
	 * 
	 * @param changed 
	 */
	protected void internalUpdateSection(boolean changed){
		createDynamicContents();
		this.reflow();
		if(changed)
			fireStateChanged();
	}
	
	public DescriptionElementBase getDescriptionElement() {
		return page.getDescriptionElement();
	}
	
	protected void fireStateChanged(){
		page.fireStateChanged();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		page.fireStateChanged();
	}
	
	public ToggleHyperlink getToggle(){
		return toggle;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.events.IExpansionListener#expansionStateChanged(org.eclipse.ui.forms.events.ExpansionEvent)
	 */
	public void expansionStateChanged(ExpansionEvent e) {
		// TODO Auto-generated method stub
		
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.forms.events.IExpansionListener#expansionStateChanging(org.eclipse.ui.forms.events.ExpansionEvent)
	 */
	public void expansionStateChanging(ExpansionEvent e) {
		// TODO Auto-generated method stub
		
	}
}
