/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.events.ExpansionEvent;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.common.AnnotatableEntity;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.AnnotationType;
import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 *
 */
public class AnnotationSection extends AbstractDescriptionDetailSection {

	private Button button_add;
	private Composite container;
	
	/**
	 * @param parent
	 * @param style
	 */
	public AnnotationSection(Composite parent,  AbstractDescriptionDetailsPage page,  int style) {
		super(parent, page, Section.TWISTIE | Section.COMPACT | style);
		
		setText("Annotations");
		setTextClient(createToolbar(this));
	}

	/**
	 * @return
	 */
	private Control createToolbar(final Section section) {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
//		IMenuService menuService = (IMenuService) EditorUtil.getService(IMenuService.class);
//		menuService.populateContributionManager(toolBarManager, "toolbar:de.md.contributions.sectionToolbar");
		
		Action addAnnotation = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				String text = "";
				AnnotationType type = AnnotationType.EDITORIAL();
				Language language = CdmStore.getDefaultLanguage();
				
				Annotation annotation = Annotation.NewInstance(text, type, language);
				getDescriptionElement().addAnnotation(annotation);
				internalUpdateSection(true);
				if(! section.isExpanded())
					section.setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addAnnotation.setToolTipText("Create a new annotation");
		toolBarManager.add(addAnnotation);
		
		return toolBarManager.createControl(section);
	}

	/*
	 * 
	 */
	public void createDynamicContents() {
				
		if(container != null)
			container.dispose();
		container = toolkit.createFormComposite(getClient(), SWT.NULL);
		
		container.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		container.setLayout(new GridLayout());
		
		Set<Annotation> annotations = null;
		if(getDescriptionElement() != null){
			annotations = getDescriptionElement().getAnnotations();
		}
		
		if(annotations == null || annotations.size() == 0){
			toolkit.createLabel(container, "No annotations yet.");
		}else{
			for(Annotation annotation : annotations){
				
				toolkit.createAnnotationComposite(container, annotation, SWT.NULL);
				
				Button button_remove = toolkit.createButton(container, "Remove", SWT.PUSH);
				button_remove.addSelectionListener(new RemoveListener(annotation));
			}
		}
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage.AbstractDescriptionDetailSection#updateSection()
	 */
	@Override
	public void updateSection() {
		internalUpdateSection(false);
	}
	
	private class RemoveListener extends SelectionAdapter{
		private Annotation annotation;
		
		public RemoveListener(Annotation annotation){
			this.annotation = annotation;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			
			AnnotatableEntity annotatedObjcet = annotation.getAnnotatedObj();
			
			annotatedObjcet.removeAnnotation(annotation);
			internalUpdateSection(true);
		}
	}
}
