/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage;

import java.util.Set;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.forms.widgets.Section;

import eu.etaxonomy.cdm.model.common.DescriptionElementSource;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;

/**
 * @author n.hoffmann
 *
 */
public class CitationSection extends AbstractDescriptionDetailSection{

	private Composite citationContainer;

	public CitationSection(Composite parent, AbstractDescriptionDetailsPage page, int style){
		super(parent, page, Section.TWISTIE | Section.COMPACT | style);
		
		setText("Citations");
		setTextClient(createToolbar(this));
	}
	
	/**
	 * @return
	 */
	private Control createToolbar(final Section section) {
		ToolBarManager toolBarManager = new ToolBarManager(SWT.FLAT);
//		IMenuService menuService = (IMenuService) EditorUtil.getService(IMenuService.class);
//		menuService.populateContributionManager(toolBarManager, "toolbar:de.md.contributions.sectionToolbar");
		
		Action addCitation = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				getDescriptionElement().addSource(DescriptionElementSource.NewInstance());
				if(! section.isExpanded())
					section.setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addCitation.setToolTipText("Create a new citation");
		toolBarManager.add(addCitation);
		
		return toolBarManager.createControl(section);
	}

	/*
	 * 
	 */
	public void createDynamicContents() {
		if(citationContainer != null)
			citationContainer.dispose();
		citationContainer = toolkit.createFormComposite(getClient(), SWT.NULL);

		citationContainer.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		citationContainer.setLayout(new GridLayout());
		
		Set<DescriptionElementSource> sources = null;
		if(getDescriptionElement() != null){
			sources = getDescriptionElement().getSources();
		}
		
		if(sources == null || sources.size() == 0){
			toolkit.createLabel(citationContainer, "No citations yet.");
		}else{
			for(DescriptionElementSource source : sources){
				toolkit.createDescriptionElementSourceComposite(citationContainer, source, SWT.NULL);
				
				Button button_remove = toolkit.createButton(citationContainer, "Remove", SWT.PUSH);
				button_remove.addSelectionListener(new RemoveListener(source));
			}
		}
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage.AbstractDescriptionDetailSection#updateSection()
	 */
	@Override
	public void updateSection() {
		internalUpdateSection(false);
	}
	
	private class RemoveListener extends SelectionAdapter{
		private DescriptionElementSource source;
		
		public RemoveListener(DescriptionElementSource source){
			this.source = source;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			DescriptionElementBase sourcedObjcet = source.getSourcedObj();
			
			sourcedObjcet.removeSource(source);
			internalUpdateSection(true);
		}
	}	
}
