/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage;

import java.net.URI;
import java.net.URISyntaxException;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.forms.ImageComposite;
import eu.etaxonomy.taxeditor.forms.TextActionComposite;

/**
 * @author nho
 *
 */
public class ImageDetailsSection extends AbstractDescriptionDetailSection implements SelectionListener{

	private TextActionComposite textAction;
	private ImageComposite imageComposite;
	private boolean changed = false;

	/**
	 * @param parent
	 * @param page
	 * @param style
	 */
	public ImageDetailsSection(Composite parent,
			AbstractDescriptionDetailsPage page, int style) {
		super(parent, page, style);
		
		GridData layoutData = (GridData) getLayoutData();
		layoutData.verticalAlignment = SWT.FILL;
		layoutData.grabExcessVerticalSpace = true;
		
		setText("Image Details"); //$NON-NLS-1$
		
		textAction = toolkit.createTextActionComposite(getClient(), "Image Uri", null, SWT.WRAP);
		textAction.addSelectionListener(this);
		
		imageComposite = toolkit.createImageComposite(getClient(), null, SWT.WRAP);
		
		// TODO
		// imageMetadataComposite = FormFactory.createImageMetaDataComposite(getClient(), null, SWT.WRAP);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.editor.descriptiontree.detailpage.AbstractDescriptionDetailSection#updateSection()
	 */
	@Override
	public void updateSection() {
		textAction.setText(getImageFile().getUri());
		try {
			imageComposite.setImageUri(new URI(CdmUtils.Nz(getImageFile().getUri())));
		} catch (URISyntaxException e) {
			// TODO remove this once cdmlib returns URIs where it seems like it does so
			e.printStackTrace();
		}
		internalUpdateSection(changed);
		changed = false;
	}
	
	private ImageFile getImageFile(){
		return (ImageFile) page.input;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		try {
			// TODO simplify once cdmlib accepts URIs where it seems like it should
			URI uri = new URI(textAction.getText());
			getImageFile().setUri(uri.toString());
			changed = true;
		} catch (URISyntaxException e1) {
			EditorUtil.warningDialog("Given image URI is not valid", "The image URI " +
					"'" + textAction.getText() + "' is not valid. Please check the spelling.");
			return;
		}
		
		updateSection();
	}
	
	public void widgetDefaultSelected(SelectionEvent e) {}
}
