/**
 * 
 */
package eu.etaxonomy.taxeditor.forms;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.common.CdmUtils;

/**
 * @author nho
 *
 */
public class TextWithLabelComposite extends FormComposite {
	protected Text text;
	
	/**
	 * @param parent
	 * @param style
	 */
	protected TextWithLabelComposite(Composite parent, String labelString, String initialText, int style) {
		super(parent, style);
		
		this.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		toolkit.createLabel(this, labelString, SWT.WRAP);
		
		text = toolkit.createText(this, "", SWT.BORDER | SWT.MULTI | SWT.WRAP);
		text.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		setText(initialText);
	}

	/**
	 * Add a ModifyListener to this composites text element
	 * 
	 * @param listener
	 */
	public void addModifyListener(ModifyListener listener){
		text.addModifyListener(listener);
	}
	
	/**
	 * Remove a ModifyListener from this composites text element
	 * 
	 * @param listener
	 */
	public void removeModifyListener(ModifyListener listener){
		text.removeModifyListener(listener);
	}

	/**
	 * Get the text of this composites text composite
	 * 
	 * @return
	 */
	public String getText() {
		return text.getText();
	}

	/**
	 * Set the text of this composites text composite
	 * 
	 * @param string
	 */
	public void setText(String string) {
		Listener[] listeners = text.getListeners(SWT.Modify);
		
		for(Listener listener :  listeners){
			text.removeListener(SWT.Modify, listener);
		}
		
		text.setText(CdmUtils.Nz(string));
		
		for(Listener listener :  listeners){
			text.addListener(SWT.Modify, listener);
		}
	}
}
