/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredContentProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.common.Extension;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.taxeditor.store.VocabularyStore;

/**
 * @author p.ciardelli
 *
 */
public class CreateExtensionDialog extends InputDialog {

	private Set<ExtensionType> extensionTypes;
	private ListViewer textViewer;
	String typeMessage = "Choose a type for the new extension";
	private String valueMessage = "Enter a value for the new extension";
	private Button okButton;
	private Set<ExtensionType> selectedExtensionTypes;
	private Text text;
	
	protected boolean hasSelection = false;
	private Extension newExtension;

	/**
	 * @param parentShell
	 * @param extensionTypes 
	 */
	protected CreateExtensionDialog(Shell shell) {
		super(shell, "Create a new extension", 
				"Choose a type and value for the new extension", null, null);
		
		extensionTypes = VocabularyStore.getExtensionTypes();
	}
	
    protected Control createDialogArea(Composite parent) {
        // create composite
		Composite composite = new Composite(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginHeight = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_MARGIN);
		layout.marginWidth = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_MARGIN);
		layout.verticalSpacing = convertVerticalDLUsToPixels(IDialogConstants.VERTICAL_SPACING);
		layout.horizontalSpacing = convertHorizontalDLUsToPixels(IDialogConstants.HORIZONTAL_SPACING);
		composite.setLayout(layout);
		composite.setLayoutData(new GridData(GridData.FILL_BOTH));
		applyDialogFont(composite);
        // create message
        if (typeMessage != null) {
            Label typeLabel = new Label(composite, SWT.WRAP);
            typeLabel.setText(typeMessage);
            GridData data = new GridData(GridData.GRAB_HORIZONTAL
                    | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                    | GridData.VERTICAL_ALIGN_CENTER);
            data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
            typeLabel.setLayoutData(data);
            typeLabel.setFont(parent.getFont());
        }
        textViewer = new ListViewer(new List(composite, SWT.SINGLE | SWT.BORDER));
        textViewer.getList().setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
        textViewer.setContentProvider(new ContentProvider());
        textViewer.setLabelProvider(new LabelProvider() {
        	/* (non-Javadoc)
        	 * @see org.eclipse.jface.viewers.LabelProvider#getText(java.lang.Object)
        	 */
        	@Override
        	public String getText(Object element) {
        		return ((ExtensionType) element).getLabel();
        	}
        });
        textViewer.setInput(extensionTypes);
        textViewer.addSelectionChangedListener(new ISelectionChangedListener () {
			public void selectionChanged(SelectionChangedEvent event) {
				hasSelection = true;
				setOkEnabled(true);
			}
		});
        // create message
        if (valueMessage != null) {
            Label valueLabel = new Label(composite, SWT.WRAP);
            valueLabel.setText(valueMessage);
            GridData data = new GridData(GridData.GRAB_HORIZONTAL
                    | GridData.GRAB_VERTICAL | GridData.HORIZONTAL_ALIGN_FILL
                    | GridData.VERTICAL_ALIGN_CENTER);
            data.widthHint = convertHorizontalDLUsToPixels(IDialogConstants.MINIMUM_MESSAGE_AREA_WIDTH);
            valueLabel.setLayoutData(data);
            valueLabel.setFont(parent.getFont());
        }
        text = new Text(composite, getInputTextStyle());
        text.setLayoutData(new GridData(GridData.GRAB_HORIZONTAL
                | GridData.HORIZONTAL_ALIGN_FILL));
        text.addModifyListener(new ModifyListener() {
            public void modifyText(ModifyEvent e) {
				setOkEnabled(true);
            }
        });
        
        applyDialogFont(composite);
        return composite;
    }
    

	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.InputDialog#createButtonsForButtonBar(org.eclipse.swt.widgets.Composite)
	 */
	protected void createButtonsForButtonBar(Composite parent) {
        // create OK and Cancel buttons by default
        okButton = createButton(parent, IDialogConstants.OK_ID,
                IDialogConstants.OK_LABEL, true);
        okButton.addMouseListener(new MouseAdapter() {
        	/* (non-Javadoc)
        	 * @see org.eclipse.swt.events.MouseAdapter#mouseUp(org.eclipse.swt.events.MouseEvent)
        	 */
        	@Override
        	public void mouseUp(MouseEvent e) {
        		close();
        	}
        });
        createButton(parent, IDialogConstants.CANCEL_ID,
                IDialogConstants.CANCEL_LABEL, false);
        setOkEnabled(false);
    }
    
	/**
	 * @param enabled
	 */
	protected void setOkEnabled(boolean enabled) {
		if (text.getText().length() > 0 && hasSelection) {
            okButton.setEnabled(enabled);			
		} else {
            okButton.setEnabled(false);
		}
	}
	    
    /* (non-Javadoc)
     * @see org.eclipse.jface.dialogs.Dialog#close()
     */
    @Override
    public boolean close() {
    	setNewExtension();
    	return super.close();
    }
    
	/**
	 * @return 
	 * 
	 */
	private void setNewExtension() {
		selectedExtensionTypes = new HashSet<ExtensionType>();
		
		ExtensionType type = (ExtensionType) ((IStructuredSelection) textViewer.getSelection()).getFirstElement();
		String value = text.getText();
		
		newExtension = Extension.NewInstance();
		newExtension.setValue(value);
		newExtension.setType(type);
	}
    
    class ContentProvider implements IStructuredContentProvider {

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IStructuredContentProvider#getElements(java.lang.Object)
		 */
		public Object[] getElements(Object inputElement) {
			return extensionTypes.toArray();
		}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#dispose()
		 */
		public void dispose() {}

		/* (non-Javadoc)
		 * @see org.eclipse.jface.viewers.IContentProvider#inputChanged(org.eclipse.jface.viewers.Viewer, java.lang.Object, java.lang.Object)
		 */
		public void inputChanged(Viewer viewer, Object oldInput, Object newInput) {}
    	
    }

	/**
	 * @return
	 */
	public Extension getNewExtension() {
		return newExtension;
	}
}
