// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.tabbed;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.forms.widgets.ExpandableComposite;
import org.eclipse.ui.forms.widgets.Section;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;

import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;

/**
 * @author n.hoffmann
 * @created 05.06.2009
 * @version 1.0
 */
public class AuthorSection extends AbstractNamePropertySection {
	private static final Logger logger = Logger.getLogger(AuthorSection.class);

	private Section authorship;
	private Composite authorshipComposite;
	
	private Section exAuthorship;
	
	private Section basionymAuthorship;
	
	private Section exBasionymAuthorship;

	private INomenclaturalAuthor combinationAuthorTeam;

	private INomenclaturalAuthor basionymAuthorTeam;

	private INomenclaturalAuthor exCombinationAuthorTeam;

	private INomenclaturalAuthor exBasionymAuthorTeam;

	private Composite exAuthorshipComposite;

	private Composite basionymAuthorshipComposite;

	private Composite exBasionymAuthorshipComposite;

	private Object sectionLayoutData;


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#createControls(org.eclipse.swt.widgets.Composite, org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage)
	 */
	@Override
	public void createControls(Composite parent,
			EditorTabbedPropertySheetPage tabbedPropertySheetPage) {
		super.createControls(parent, tabbedPropertySheetPage);

		sectionLayoutData = new GridData(SWT.FILL, SWT.TOP, true, false, 2, 1);
		
		authorship = getWidgetFactory().createSection(composite, ExpandableComposite.TREE_NODE | ExpandableComposite.CLIENT_INDENT);
		authorshipComposite = getWidgetFactory().createComposite(authorship);
		authorship.setClient(authorshipComposite);
		authorship.setLayoutData(sectionLayoutData);
		
		exAuthorship = getWidgetFactory().createSection(composite, ExpandableComposite.TREE_NODE | ExpandableComposite.CLIENT_INDENT);
		exAuthorshipComposite = getWidgetFactory().createComposite(exAuthorship);
		exAuthorship.setClient(exAuthorshipComposite);
		exAuthorship.setLayoutData(sectionLayoutData);
		
		basionymAuthorship = getWidgetFactory().createSection(composite, ExpandableComposite.TREE_NODE | ExpandableComposite.CLIENT_INDENT);
		basionymAuthorshipComposite = getWidgetFactory().createComposite(basionymAuthorship);
		basionymAuthorship.setClient(basionymAuthorshipComposite);
		basionymAuthorship.setLayoutData(sectionLayoutData);
		
		exBasionymAuthorship = getWidgetFactory().createSection(composite, ExpandableComposite.TREE_NODE | ExpandableComposite.CLIENT_INDENT);
		exBasionymAuthorshipComposite = getWidgetFactory().createComposite(exBasionymAuthorship);
		exBasionymAuthorship.setClient(exBasionymAuthorshipComposite);
		exBasionymAuthorship.setLayoutData(sectionLayoutData);
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.tabbed.AbstractPropertySection#refresh()
	 */
	@Override
	public void refresh() {
		super.refresh();
		
		section.setText("Author");
		
		cacheText.removeModifyListener(listener);
		cacheText.setText(name.getAuthorshipCache());
		cacheText.addModifyListener(listener);
		editCacheButton.setSelection(name.isProtectedAuthorshipCache());
		
		authorship.setText("Authorship");
		exAuthorship.setText("Ex-Authorship");
		basionymAuthorship.setText("Basionym Authorship");
		exBasionymAuthorship.setText("Ex-Basionym Authorship");
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.propertysheet.tabbed.AbstractNamePropertySection#setInput(org.eclipse.ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
	 */
	@Override
	public void setInput(IWorkbenchPart part, ISelection selection) {
		super.setInput(part, selection);
		
		combinationAuthorTeam = name.getCombinationAuthorTeam();
		exCombinationAuthorTeam = name.getExCombinationAuthorTeam();
		basionymAuthorTeam = name.getBasionymAuthorTeam();
		exBasionymAuthorTeam = name.getExBasionymAuthorTeam();
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.propertysheet.tabbed.AbstractNamePropertySection#toggleCacheEditing()
	 */
	@Override
	protected void toggleCacheEditing() {
		super.toggleCacheEditing();
		
		if(name.isProtectedAuthorshipCache()){
			name.setProtectedAuthorshipCache(false);
		}else{
			name.setProtectedAuthorshipCache(true);
		}
	}	
}
