/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type;

import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.DialogCellEditor;
import org.eclipse.jface.window.Window;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.views.properties.PropertyDescriptor;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.taxeditor.editor.EditorUtil;
import eu.etaxonomy.taxeditor.model.NameHelper;
import eu.etaxonomy.taxeditor.propertysheet.type.wizard.TypeListWizard;

/**
 * Property sheet descriptor that pops up a dialog window
 *  returning a set.
 *  
 * @author p.ciardelli
 * @created 19.05.2008
 * @version 1.0
 */
public class TypePropertyDescriptor extends PropertyDescriptor {
	
	private TaxonNameBase<?, ?> name;
	
	public TypePropertyDescriptor(Object id, String displayName, TaxonNameBase<?, ?> name) {
		super(id, displayName);
		this.name = name;
	}

    public CellEditor createPropertyEditor(Composite parent) {
        CellEditor editor = new DialogCellEditor(parent) {

			@Override
			protected Object openDialogBox(
					Control cellEditorWindow) {
				
				Wizard wizard = new TypeListWizard(name);

				// Instantiates the wizard container with the wizard and opens it
				WizardDialog dialog = new WizardDialog(EditorUtil.getShell(), wizard);
				dialog.create();
				int returnCode = dialog.open();
				
				if (returnCode == Window.OK) {
					if (NameHelper.isNameSupraSpecific(name)) {
						return new TypeCollectionPropertySource(name, name.getNameTypeDesignations());
					} else {
						return new TypeCollectionPropertySource(name, name.getSpecimenTypeDesignations());
					}
				} else {
					return null;					
				}
			}
        };
        if (getValidator() != null) {
			editor.setValidator(getValidator());
		}
        return editor;
    }
}