/**
* Copyright (C) 2009 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type.wizard;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.apache.log4j.Logger;
import org.eclipse.core.databinding.observable.list.WritableList;
import org.eclipse.core.runtime.Assert;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.editor.name.NameSelectComposite;
import eu.etaxonomy.taxeditor.editor.reference.ReferenceSelectComposite;
import eu.etaxonomy.taxeditor.preference.PreferencesUtil;

/**
 * @author p.ciardelli
 * @created 09.03.2009
 * @version 1.0
 */
public class ChooseNameTypeWizardPage extends WizardPage {
	private static Logger logger = Logger
			.getLogger(ChooseNameTypeWizardPage.class);

	private TaxonNameBase<?, ?> name;	
	private NameTypeDesignation typeDesignation;
	private TaxonNameBase<?, ?> savedName;
	
	private Text txtName;
	private Button btnRejected;
	private Button btnConserved;
	private Button btnLecto;
	private Button btnNotDesignated;

	private WritableList typeDesignationsList;

	private ReferenceSelectComposite referenceComposite;

	private NameSelectComposite nameComposite;

	private boolean nameSelected;

	/**
	 * @param typeDesignation
	 * @param name
	 * @param typeDesignationsList 
	 */
	public ChooseNameTypeWizardPage(TypeDesignationBase<?> typeDesignation,
			TaxonNameBase<?, ?> name, WritableList typeDesignationsList) {
		super("");
		
		Assert.isTrue(typeDesignation == null || typeDesignation instanceof NameTypeDesignation);
		
		this.typeDesignation = (NameTypeDesignation) typeDesignation;
		this.name = name;
		this.typeDesignationsList = typeDesignationsList;
		
		setTitle("Create or edit type designation");
		setDescription("Create or edit type designation for '" + name.getTitleCache() + "\".");
		logger.trace(this.getClass().getSimpleName() + " created.");
	}
	
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		
		// Create composite for popup dialog	
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new GridLayout());
		setControl(container);
			
		// Create name input composite
		nameComposite = new NameSelectComposite(container);
		nameComposite.addPropertyChangeListener(new PropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent evt) {
				setName();
			}
		});
		
		if (typeDesignation != null) {
			nameComposite.setName(typeDesignation.getTypeName());
		}
		
		// Create 2-columned composite for name
		Composite booleansComposite = new Composite(container, SWT.NULL);
		booleansComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		GridLayout gridLayout2 = new GridLayout();
		gridLayout2.numColumns = 2;
		gridLayout2.marginHeight = 15;
		gridLayout2.marginWidth = 0;
		booleansComposite.setLayout(gridLayout2);
		
		btnRejected = new Button(booleansComposite, SWT.CHECK);
		btnRejected.setText("Is rejected type");
		btnRejected.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnConserved = new Button(booleansComposite, SWT.CHECK);
		btnConserved.setText("Is conserved type");
		btnConserved.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnLecto = new Button(booleansComposite, SWT.CHECK);
		btnLecto.setText("Is lecto type");
		btnLecto.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		btnNotDesignated = new Button(booleansComposite, SWT.CHECK);
		btnNotDesignated.setText("Is not designated");
		btnNotDesignated.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
		
		// Set any boolean values
		if (typeDesignation != null) {
			if (typeDesignation.isRejectedType()) {
				btnRejected.setSelection(true);
			}
			if (typeDesignation.isConservedType()) {
				btnConserved.setSelection(true);
			}
			if (typeDesignation.isLectoType()) {
				btnLecto.setSelection(true);
			}
			if (typeDesignation.isNotDesignated()) {
				btnNotDesignated.setSelection(true);
			}
		}
				
		// Create reference input composite
		referenceComposite = new ReferenceSelectComposite(container);
		
		// Set text to reference text if exists
		if (typeDesignation != null && typeDesignation.getCitation() != null) {
			referenceComposite.setMicroReference(typeDesignation.getCitationMicroReference());
			if (typeDesignation.getCitation() != null) {
				referenceComposite.setReference(typeDesignation.getCitation());
			}
		}
	}
	
	/**
	 */
	private void setName() {
		savedName = nameComposite.getName();
		nameSelected = (savedName != null);
		updatePage();
	}
		
	/**
	 * 
	 */
	protected void clearName() {
		savedName = null;
		txtName.setText("");				
	}
		
	@Override
	public boolean canFlipToNextPage() {
		return isPageComplete();
	}

	public boolean isPageComplete() {
		return nameComposite.getName() != null;
	}
	
	private void updatePage() {
		getWizard().getContainer().updateButtons();
	}
	
    public void setPageComplete(boolean complete) {
        super.setPageComplete(complete);
        
        if (complete) {
        	
        	TaxonNameBase<?, ?> typeName = null;
        	if (savedName != null) {
        		typeName = savedName;
        	} else {
        		if (!txtName.getText().equals("")) {
        			typeName = PreferencesUtil.getPreferredNomenclaturalCode().getNewTaxonNameInstance(null);
        			typeName.setTitleCache(txtName.getText());
        		}
        	}
        	
        	ReferenceBase<?> citation = referenceComposite.getReference();
        	String citationMicroReference = referenceComposite.getMicroReference();
        	        	
        	if (typeDesignation == null) {
        		typeDesignation = new TemporaryNameTypeDesignation(typeName, citation, citationMicroReference,
        				null, btnRejected.getSelection(), btnConserved.getSelection(), btnLecto.getSelection(), btnNotDesignated.getSelection());
//        		typeDesignation.setLectoType(btnLecto.getSelection());
        	} else {
        		typeDesignation.setRejectedType(btnRejected.getSelection());
        		typeDesignation.setConservedType(btnConserved.getSelection());
//        		typeDesignation.setLectoType(btnLecto.getSelection());
        		typeDesignation.setNotDesignated(btnNotDesignated.getSelection());
        		typeDesignation.setCitation(citation);
        		typeDesignation.setCitationMicroReference(citationMicroReference);
        	}
        	
        	typeDesignationsList.remove(typeDesignation);
    		typeDesignationsList.add(typeDesignation);
        }
    }
    
	public class TemporaryNameTypeDesignation extends
			NameTypeDesignation {
				
		/**
		 * 
		 */
		private static final long serialVersionUID = 7309597024482795166L;

		public TemporaryNameTypeDesignation(TaxonNameBase<?, ?> typeName, ReferenceBase<?> citation, String citationMicroReference,
				String originalNameString, boolean isRejectedType, boolean isConservedType, boolean isLectoType, boolean isNotDesignated) {
			super(typeName, citation, citationMicroReference, originalNameString, null/*TODO add type status*/,
					isRejectedType, isConservedType,  /*isLectoType,*/ isNotDesignated);
		}
	}
}
