// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.newWizards;

import org.apache.log4j.Logger;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.taxon.ITreeNode;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.cdm.model.taxon.TaxonomicTree;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredNameSelectionDialog;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredTaxonNodeSelectionDialog;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredTaxonSelectionDialog;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredTaxonomicTreeSelectionDialog;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.parser.ParseHandler;

/**
 * @author n.hoffmann
 * @created Sep 15, 2009
 * @version 1.0
 */
public class NewTaxonNodeWizardPage extends WizardPage implements ModifyListener {

	private static final Logger logger = Logger
			.getLogger(NewTaxonNodeWizardPage.class);
	
	private static final String BUTTON_TEXT_BROWSE = "Browse";
	
	private TaxonomicTree taxonomicTree;
	
	private ParseHandler parseHandler;
	
	private TaxonNode parentTaxonNode;
	
	private TaxonBase taxonBase;
	
	private boolean reuseExistingTaxon = false;
	
	private TaxonNameBase taxonName;
	
	private boolean openInEditor = true;

	private Button button_openInEditor;

	private Text text_parentTaxonNode;

	private Text text_reuseTaxon;

	private Text text_taxonName;

	private Text text_taxonomicTree;

	private Button button_taxonName;

	private Button button_reuseTaxon;

	private Button button_toggleReuseTaxon;
	
	/**
	 * @param pageName
	 */
	protected NewTaxonNodeWizardPage(IStructuredSelection selection) {
		super("New Taxon Node");
		
		if(selection != null && selection.size() == 1){
			if(selection.getFirstElement() instanceof ITreeNode){
				ITreeNode parentNode = (ITreeNode) selection.getFirstElement();			
				
				if(parentNode instanceof TaxonNode){
					parentTaxonNode = (TaxonNode) parentNode;
					taxonomicTree = parentTaxonNode.getTaxonomicTree();
				}else{
					taxonomicTree = (TaxonomicTree) parentNode;
				}
			}
		}else{
			setPageComplete(false);
		}
		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.NULL);
		
		GridLayout layout = new GridLayout(3, false);
		composite.setLayout(layout);
		
		// taxonomic tree
		Label label_taxonomicTree = new Label(composite, SWT.NULL);
		label_taxonomicTree.setText("Taxonomic Tree");
		
		text_taxonomicTree = new Text(composite, SWT.BORDER | SWT.SINGLE);
		text_taxonomicTree.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		text_taxonomicTree.setEnabled(false);
		text_taxonomicTree.addModifyListener(this);
		text_taxonomicTree.setText(taxonomicTree != null ? taxonomicTree.getTitleCache() : "");
		
		Button button_taxonomicTree = new Button(composite, SWT.PUSH);
		button_taxonomicTree.setText(BUTTON_TEXT_BROWSE);
		button_taxonomicTree.addSelectionListener(new TaxonomicTreeSelector());
		
		
		// parental node
		Label label_parentTaxonNode = new Label(composite, SWT.NULL);
		label_parentTaxonNode.setText("Parent Taxon Node");
		
		text_parentTaxonNode = new Text(composite, SWT.BORDER | SWT.SINGLE);
		text_parentTaxonNode.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		text_parentTaxonNode.setEnabled(false);
		text_parentTaxonNode.setText(parentTaxonNode != null ? parentTaxonNode.getTaxon().getName().getTitleCache() : "");
		
		Button button_parentTaxonNode = new Button(composite, SWT.PUSH);
		button_parentTaxonNode.setText(BUTTON_TEXT_BROWSE);
		button_parentTaxonNode.addSelectionListener(new TaxonNodeSelector());
		
		// existing taxon
		Composite container_label_reuseTaxon = new Composite(composite, SWT.NULL);
		container_label_reuseTaxon.setLayout(new GridLayout(2, false));
		
		button_toggleReuseTaxon = new Button(container_label_reuseTaxon, SWT.CHECK);
		button_toggleReuseTaxon.addSelectionListener(new ToggleReuseTaxonSelector());
		button_toggleReuseTaxon.setSelection(reuseExistingTaxon);
		
		Label label_reuseTaxon = new Label(container_label_reuseTaxon, SWT.NULL);
		label_reuseTaxon.setText("Reuse Existing Taxon");
		label_reuseTaxon.setEnabled(false);
		
		text_reuseTaxon = new Text(composite, SWT.BORDER | SWT.SINGLE);
		text_reuseTaxon.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		text_reuseTaxon.setEnabled(false);
	
		
		button_reuseTaxon = new Button(composite, SWT.PUSH);
		button_reuseTaxon.setText(BUTTON_TEXT_BROWSE);
		button_reuseTaxon.addSelectionListener(new TaxonSelector());
		button_reuseTaxon.setEnabled(false);
		
		// name
		Label label_taxonName = new Label(composite, SWT.NULL);
		label_taxonName.setText("Taxon Name");
		
		text_taxonName = new Text(composite, SWT.BORDER | SWT.SINGLE);
		text_taxonName.addModifyListener(this);
		text_taxonName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true));
		
		button_taxonName = new Button(composite, SWT.PUSH);
		button_taxonName.setText(BUTTON_TEXT_BROWSE);
		button_taxonName.addSelectionListener(new TaxonNameSelector());
		
		// open in editor
		Composite container_openInEditor = new Composite(composite, SWT.NULL);
		container_openInEditor.setLayout(new GridLayout(2, false));
		button_openInEditor = new Button(container_openInEditor, SWT.CHECK);
		button_openInEditor.addSelectionListener(new OpenInEditorSelector());
		button_openInEditor.setSelection(openInEditor);
		
		Label label_openInEditor = new Label(container_openInEditor, SWT.NULL);
		label_openInEditor.setText("Open Taxon in Editor");
				
		// initialize the parse handler
		parseHandler = ParseHandler.NewInstance(text_taxonName, taxonName);
		
		// set focus to name field
		text_taxonName.setFocus();
		
		setControl(composite);
	}
	
	/**
	 * @return the taxon
	 */
	public TaxonBase getTaxon() {
		return taxonBase;
	}

	/**
	 * @return the name
	 */
	public TaxonNameBase getTaxonName() {
		return taxonName;
	}
	
	/**
	 * @return
	 */
	public ITreeNode getParentNode() {
		return parentTaxonNode != null ? parentTaxonNode : taxonomicTree;
	}
	
	public boolean openInEditor(){
		return openInEditor;
	}
	
	private void updatePage() {
		getWizard().getContainer().updateButtons();
	}
	
	/*
	 * SELECTION ADAPTERS
	 */
	
	private class TaxonomicTreeSelector extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			taxonomicTree = FilteredTaxonomicTreeSelectionDialog.selectTaxonomicTree(NavigationUtil.getShell());
			if (taxonomicTree != null) {
				text_taxonomicTree.setText(taxonomicTree.getTitleCache());
				text_parentTaxonNode.setText("");
				parentTaxonNode = null;
			}
			setPageComplete(true);
		}
	}
	
	private class TaxonNodeSelector extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			parentTaxonNode = FilteredTaxonNodeSelectionDialog.selectTaxonNode(NavigationUtil.getShell(), "Select parent", null, null);
			if (parentTaxonNode != null) {
				text_parentTaxonNode.setText(parentTaxonNode.getTaxon().getTitleCache());
				text_taxonomicTree.setText(parentTaxonNode.getTaxonomicTree().getTitleCache());
			}
			text_taxonomicTree.setText(parentTaxonNode.getTaxonomicTree().getTitleCache());
			setPageComplete(true);
		}
		
	}
	
	private class TaxonSelector extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			taxonBase = FilteredTaxonSelectionDialog.selectTaxon(NavigationUtil.getShell());
			if (taxonBase != null) {
				text_reuseTaxon.setText(taxonBase.getTitleCache());
			}
		}
	}
	
	private class TaxonNameSelector extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			taxonName = FilteredNameSelectionDialog.selectName(NavigationUtil.getShell(), null);
			if (taxonName != null) {
				text_taxonName.setText(taxonName.getTitleCache());
			}
		}
	}
	
	private class OpenInEditorSelector extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			openInEditor = button_openInEditor.getSelection();
		}
	}
	
	private class ToggleReuseTaxonSelector extends SelectionAdapter{
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			super.widgetSelected(e);
			
			if(button_toggleReuseTaxon.getSelection()){
				// enable reuse Taxon
				text_reuseTaxon.setEnabled(true);
				button_reuseTaxon.setEnabled(true);
				// disable name
				text_taxonName.setEnabled(false);
				button_taxonName.setEnabled(false);
			}else{
				// disable reuse Taxon
				text_reuseTaxon.setEnabled(false);
				button_reuseTaxon.setEnabled(false);
				// enable name
				text_taxonName.setEnabled(true);
				button_taxonName.setEnabled(true);
			}
			
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.ModifyListener#modifyText(org.eclipse.swt.events.ModifyEvent)
	 */
	public void modifyText(ModifyEvent e) {
		if (e.getSource().equals(text_taxonName)) {
			taxonName = parseHandler.parse();
		}
		updatePage();
	}	
}
