// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.io;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.Assert;

import eu.etaxonomy.cdm.api.application.CdmApplicationController;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.io.berlinModel.in.BerlinModelImportConfigurator;
import eu.etaxonomy.cdm.io.common.CdmDefaultImport;
import eu.etaxonomy.cdm.io.common.IImportConfigurator;
import eu.etaxonomy.cdm.io.excel.taxa.NormalExplicitImportConfigurator;
import eu.etaxonomy.cdm.io.jaxb.JaxbImportConfigurator;
import eu.etaxonomy.cdm.io.reference.endnote.in.EndnoteImportConfigurator;
import eu.etaxonomy.cdm.io.tcsxml.in.TcsXmlImportConfigurator;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author n.hoffmann
 * @created Sep 11, 2009
 * @version 1.0
 */
public class ImportHandler extends AbstractIOHandler {

	private static final Logger logger = Logger
			.getLogger(ImportHandler.class);
	
	/**
	 * @param applicationController
	 */
	private ImportHandler(
			CdmApplicationController applicationController) {
		super(applicationController);
	}
	
	public static ImportHandler NewInstance(CdmApplicationController applicationController){
		return new ImportHandler(applicationController);
	}
	
	/**
	 * Use when importing data into the current application context
	 * 
	 * @param configurator
	 * @return
	 */
	public void doImport(final IImportConfigurator configurator){
		Assert.isNotNull(configurator, "Configuration may not be null");
		
		// terminate any open transactions
		IConversationEnabled activePart = (IConversationEnabled) StoreUtil.getActivePage().getActivePart();
		activePart.getConversationHolder().commit(false);
		
		
		CdmDefaultImport<IImportConfigurator> importer = new CdmDefaultImport<IImportConfigurator>();

		importer.setCdmAppController(applicationController);
		
		
		try{
			importer.invoke(configurator);
		}catch(RuntimeException e){
			StoreUtil.errorDialog("Error importing data", "An error occured while" +
					"importing from source '" + configurator.getSourceNameString() + "'.\n" +
							"Please check error log for details.");
			logger.error("Error importing data", e);
		}
		
		// restarting transaction and committing it to trigger change listener
		// TODO verify correct behaviour
		activePart.getConversationHolder().startTransaction();
		activePart.getConversationHolder().commit();
	}

	private IImportConfigurator getConfigurator(TYPE type){
		Assert.isNotNull(type, "Type should not be null");
		
		switch(type){
			case BerlinModel:
				return BerlinModelImportConfigurator.NewInstance(null, null);	
			case Jaxb:
				return JaxbImportConfigurator.NewInstance(null, null);
			case Tcs:
				return TcsXmlImportConfigurator.NewInstance(null, null);
			case Endnote:
				return EndnoteImportConfigurator.NewInstance(null, null);
			case Excel_Taxa:
				return NormalExplicitImportConfigurator.NewInstance(null, null, null);
			default:
				StoreUtil.notImplementedMessage();
				throw new IllegalArgumentException("Import not supported yet");
		}
	}
	
	/** @return a <code>BerlinModelImportConfigurator</code>*/
	public final BerlinModelImportConfigurator BerlinModelConfigurator() { return (BerlinModelImportConfigurator) getConfigurator(TYPE.BerlinModel);	}
	/** @return a <code>JaxbImportConfigurator</code>*/
	public final JaxbImportConfigurator JaxbConfigurator() { return (JaxbImportConfigurator) getConfigurator(TYPE.Jaxb);	}
	/** @return a <code>TcsXmlImportConfigurator</code>*/
	public final TcsXmlImportConfigurator TcsConfigurator() { return (TcsXmlImportConfigurator) getConfigurator(TYPE.Tcs);	}
	/** @return an <code>EndnoteImportConfigurator</code> */
	public final EndnoteImportConfigurator EndnoteImportConfigurator() { return (EndnoteImportConfigurator) getConfigurator(TYPE.Endnote);   }
	/** @return a <code>NormalExplicitImportConfigurator</code>*/
	public final NormalExplicitImportConfigurator NormalExplicitImportConfigurator() { return (NormalExplicitImportConfigurator) getConfigurator(TYPE.Excel_Taxa); }
	
	
}
