// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 02.07.2009
 * @version 1.0
 */
public class DeleteUserOperation extends AbstractPostOperation {

	private static final Logger logger = Logger
			.getLogger(DeleteUserOperation.class);

	private UUID userUuid;

	private User user;

	/**
	 * @param label
	 * @param undoContext
	 * @param parentNodeUuid
	 * @param postOperationEnabled
	 */
	public DeleteUserOperation(String label, IUndoContext undoContext,
			UUID userUuid, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, postOperationEnabled);
		
		this.userUuid = userUuid;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		user = CdmStore.getUserService().find(userUuid);
		CdmStore.getUserService().delete(user);
		conversation.commit();
		
		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		ConversationHolder conversation = CdmStore.NewTransactionalConversation();
		// store uuid for consecutive undo/redo steps
		userUuid = CdmStore.getUserService().save(user);
		conversation.commit();
		
		return postExecute(null);
	}
}
