/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.preference;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author p.ciardelli
 * @created 16.09.2008
 * @version 1.0
 */
public class NomenclaturalCodePreferences extends PreferencePage implements
		IWorkbenchPreferencePage {

	public static final String PLUGIN_ID = "eu.etaxonomy.taxeditor.preferences.nomenclatural";
	
	private NomenclaturalCode preferredCode;

	private Map<NomenclaturalCode, Button> buttons = new HashMap<NomenclaturalCode, Button>();
		
	public NomenclaturalCodePreferences() {
		super();
		setDescription("Choose which nomenclatural code you would like to use for scientific names unless otherwise specified.");
	}

	/**
	 * Create contents of the preference page
	 * @param parent
	 */
	@Override
	public Control createContents(Composite parent) {
		
		Composite container = new Composite(parent, SWT.NULL);
		container.setLayout(new GridLayout());
		
		for (final NomenclaturalCode code : NomenclaturalCodeHelper.getSupportedCodes()) {
			Button button = new Button(container, SWT.RADIO);
			button.setText(NomenclaturalCodeHelper.getDescription(code));
			button.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					preferredCode = code;
				}
			});
			buttons.put(code, button);
		}
		
		preferredCode = PreferencesUtil.getPreferredNomenclaturalCode();
		if (preferredCode == null) {
			preferredCode = NomenclaturalCodeHelper.getDefaultCode();
			PreferencesUtil.setPreferredNomenclaturalCode(preferredCode);
		}
		setButton(preferredCode);		
		
		return container;
	}

	/**
	 * Initialize the preference page
	 */
	public void init(IWorkbench workbench) {
		setPreferenceStore(TaxeditorStorePlugin.getDefault().getPreferenceStore());
	}
	
	protected void performDefaults() {
		setButton(NomenclaturalCodeHelper.getDefaultCode());
	}
	
	public boolean performOk() {
		PreferencesUtil.setPreferredNomenclaturalCode(preferredCode);
		return true;
	}
	
	private void setButton(NomenclaturalCode preferredCode) {
		this.preferredCode = preferredCode;
		for (NomenclaturalCode code : buttons.keySet()) {
			buttons.get(code).setSelection(code.equals(preferredCode));
		}
	}
}
