// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.user.view;

import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.GroupMarker;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.ViewPart;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.User;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created 01.07.2009
 * @version 1.0
 */
public class UserManagerView extends ViewPart implements IConversationEnabled, IPostOperationEnabled{
	private static final Logger logger = Logger
			.getLogger(UserManagerView.class);
	
	public static String ID = "eu.etaxonomy.taxeditor.store.userManagerView";
	
	private ListViewer listViewer;

	private ConversationHolder conversation;

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	@Override
	public void createPartControl(Composite parent) {
		
		conversation = CdmStore.NewTransactionalConversation();
		
		FillLayout fillLayout = new FillLayout();
		fillLayout.marginWidth = 0;
		fillLayout.marginHeight = 0;
		fillLayout.type = SWT.VERTICAL;	
		parent.setLayout(fillLayout);
		
		listViewer = new ListViewer(parent, SWT.MULTI | SWT.H_SCROLL
				| SWT.V_SCROLL | SWT.FULL_SELECTION);
		
		MenuManager menuMgr = new MenuManager();
		menuMgr.add(new GroupMarker(IWorkbenchActionConstants.MB_ADDITIONS));
		getSite().registerContextMenu(menuMgr, listViewer);

		Control control = listViewer.getControl();
		Menu menu = menuMgr.createContextMenu(control);
		control.setMenu(menu);	
		
		listViewer.setContentProvider(new UserManagerContentProvider());
		listViewer.setLabelProvider(new UserManagerLabelProvider());
				
		listViewer.setInput(getAllUser());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	@Override
	public void setFocus() {
		conversation.bind();
		listViewer.getControl().setFocus();
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		listViewer.setInput(getAllUser());
		listViewer.refresh();
		
		return true;
	}
	
	/**
	 * 
	 * @return
	 */
	private List<User> getAllUser(){
		return CdmStore.getUserService().list(null, 100000, 0, null, null);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	public void update(CdmDataChangeMap changeEvents) {
		// data changes can only be generate here at the moment
	}
	
}
